'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var CALL_API = Symbol('CALL_API');
var REDUCER_PATH = 'api_calls';
var ACTION_FETCH_START = '@@api/FETCH_START';
var ACTION_FETCH_COMPLETE = '@@api/FETCH_COMPLETE';
var ACTION_FETCH_FAILURE = '@@api/FETCH_FAILURE';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) {
  return typeof obj;
} : function (obj) {
  return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
};





var asyncGenerator = function () {
  function AwaitValue(value) {
    this.value = value;
  }

  function AsyncGenerator(gen) {
    var front, back;

    function send(key, arg) {
      return new Promise(function (resolve, reject) {
        var request = {
          key: key,
          arg: arg,
          resolve: resolve,
          reject: reject,
          next: null
        };

        if (back) {
          back = back.next = request;
        } else {
          front = back = request;
          resume(key, arg);
        }
      });
    }

    function resume(key, arg) {
      try {
        var result = gen[key](arg);
        var value = result.value;

        if (value instanceof AwaitValue) {
          Promise.resolve(value.value).then(function (arg) {
            resume("next", arg);
          }, function (arg) {
            resume("throw", arg);
          });
        } else {
          settle(result.done ? "return" : "normal", result.value);
        }
      } catch (err) {
        settle("throw", err);
      }
    }

    function settle(type, value) {
      switch (type) {
        case "return":
          front.resolve({
            value: value,
            done: true
          });
          break;

        case "throw":
          front.reject(value);
          break;

        default:
          front.resolve({
            value: value,
            done: false
          });
          break;
      }

      front = front.next;

      if (front) {
        resume(front.key, front.arg);
      } else {
        back = null;
      }
    }

    this._invoke = send;

    if (typeof gen.return !== "function") {
      this.return = undefined;
    }
  }

  if (typeof Symbol === "function" && Symbol.asyncIterator) {
    AsyncGenerator.prototype[Symbol.asyncIterator] = function () {
      return this;
    };
  }

  AsyncGenerator.prototype.next = function (arg) {
    return this._invoke("next", arg);
  };

  AsyncGenerator.prototype.throw = function (arg) {
    return this._invoke("throw", arg);
  };

  AsyncGenerator.prototype.return = function (arg) {
    return this._invoke("return", arg);
  };

  return {
    wrap: function (fn) {
      return function () {
        return new AsyncGenerator(fn.apply(this, arguments));
      };
    },
    await: function (value) {
      return new AwaitValue(value);
    }
  };
}();













var defineProperty = function (obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
};

var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};

var get = function get(object, property, receiver) {
  if (object === null) object = Function.prototype;
  var desc = Object.getOwnPropertyDescriptor(object, property);

  if (desc === undefined) {
    var parent = Object.getPrototypeOf(object);

    if (parent === null) {
      return undefined;
    } else {
      return get(parent, property, receiver);
    }
  } else if ("value" in desc) {
    return desc.value;
  } else {
    var getter = desc.get;

    if (getter === undefined) {
      return undefined;
    }

    return getter.call(receiver);
  }
};











var objectWithoutProperties = function (obj, keys) {
  var target = {};

  for (var i in obj) {
    if (keys.indexOf(i) >= 0) continue;
    if (!Object.prototype.hasOwnProperty.call(obj, i)) continue;
    target[i] = obj[i];
  }

  return target;
};





var set = function set(object, property, value, receiver) {
  var desc = Object.getOwnPropertyDescriptor(object, property);

  if (desc === undefined) {
    var parent = Object.getPrototypeOf(object);

    if (parent !== null) {
      set(parent, property, value, receiver);
    }
  } else if ("value" in desc && desc.writable) {
    desc.value = value;
  } else {
    var setter = desc.set;

    if (setter !== undefined) {
      setter.call(receiver, value);
    }
  }

  return value;
};

var assertTypes = (function (value, types, msg) {
  if (!types.test(typeof value === "undefined" ? "undefined" : _typeof(value))) throw new Error(msg);
});

var validateApi = function validateApi(api) {
  assertTypes(api.name, /string/, 'name must be a string');

  assertTypes(api.endpoint, /string|function/, 'endpoint must be either a string or a function');
};

var defaultAdapter = (function (_ref) {
  var endpoint = _ref.endpoint;
  var others = objectWithoutProperties(_ref, ["endpoint"]);

  return fetch(endpoint, others);
});

var makeStartErrorAction = function makeStartErrorAction(api) {
  return function (error) {
    return {
      type: ACTION_FETCH_START,
      error: true,
      payload: _extends({
        error: error
      }, api)
    };
  };
};

var makeStartAction = function makeStartAction(api) {
  return function () {
    return {
      type: ACTION_FETCH_START,
      payload: _extends({}, api, {
        requestedAt: Date.now()
      })
    };
  };
};

var makeSuccessAction = function makeSuccessAction(api) {
  return function (json) {
    return {
      type: ACTION_FETCH_COMPLETE,
      payload: _extends({}, api, {
        json: json,
        respondedAt: Date.now()
      })
    };
  };
};

var makeFailureAction = function makeFailureAction(api) {
  return function (json) {
    return {
      type: ACTION_FETCH_FAILURE,
      payload: _extends({}, api, {
        json: json,
        respondedAt: Date.now()
      })
    };
  };
};

var middleware = (function () {
  var adapter = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : defaultAdapter;
  return function (_ref) {
    var dispatch = _ref.dispatch;
    var getState = _ref.getState;
    return function (next) {
      return function (action) {
        if (!action || !action[CALL_API]) {
          next(action);
          return;
        }

        var api = action[CALL_API];

        try {
          var _ret = function () {

            var finalApi = Object.keys(api).reduce(function (obj, key) {
              return _extends({}, obj, defineProperty({}, key, typeof api[key] === 'function' ? api[key](getState()) : api[key]));
            }, {});

            validateApi(finalApi);

            dispatch(makeStartAction(finalApi)());

            return {
              v: new Promise(function (resolve, reject) {
                var responsePromise = adapter(finalApi);

                responsePromise.then(function (resp) {
                  if (resp.ok) {
                    return resp.json().then(makeSuccessAction(finalApi));
                  }

                  return resp.json().then(makeFailureAction(finalApi));
                }).then(dispatch).then(resolve);
              })
            };
          }();

          if ((typeof _ret === 'undefined' ? 'undefined' : _typeof(_ret)) === "object") return _ret.v;
        } catch (error) {
          dispatch(makeStartErrorAction(api)(error));
          return;
        }
      };
    };
  };
});

var get$1 = (function (array, defaulValue) {
  return function (state) {
    var finalValue = array.reduce(function (value, nextProp) {
      if (typeof value === 'undefined' || value === null) {
        return;
      }
      return value[nextProp];
    }, state);

    if (typeof finalValue === 'undefined') {
      return defaulValue;
    }

    return finalValue;
  };
});

var makeFetchAction = (function (apiName, apiConfigFn) {
  var selectorDescriptor = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

  var actionCreator = function actionCreator() {
    return defineProperty({}, CALL_API, _extends({}, apiConfigFn.apply(undefined, arguments), {
      name: apiName
    }));
  };

  var isFetchingSelector = get$1([REDUCER_PATH, apiName, 'isFetching'], false);
  var isInvalidatedSelector = get$1([REDUCER_PATH, apiName, 'isInvalidated'], false);
  var dataSelector = get$1([REDUCER_PATH, apiName, 'data'], null);
  var errorSelector = get$1([REDUCER_PATH, apiName, 'error'], null);

  return {
    actionCreator: actionCreator,
    isFetchingSelector: isFetchingSelector,
    isInvalidatedSelector: isInvalidatedSelector,
    dataSelector: dataSelector,
    errorSelector: errorSelector
  };
});

var handleActions = (function (handlers) {
  return function () {
    var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    var action = arguments[1];

    var handler = handlers[action.type];

    if (typeof handler !== 'function') {
      return state;
    }

    return handler(state, action);
  };
});

var _handleActions;

var getName = function getName(action) {
  return action.payload.name;
};
var getRequestedAt = function getRequestedAt(action) {
  return action.payload.requestedAt;
};
var getRespondedAt = function getRespondedAt(action) {
  return action.payload.respondedAt;
};
var getJSONResponse = function getJSONResponse(action) {
  return action.payload.json;
};
var getError = function getError(action) {
  return action.payload.error;
};

var reducer = handleActions((_handleActions = {}, defineProperty(_handleActions, ACTION_FETCH_START, function (state, action) {
  var apiName = getName(action);
  var api = state[apiName];
  var next = _extends({}, api, {
    lastRequest: getRequestedAt(action),
    isFetching: !action.error,
    isInvalidated: true,
    error: action.error ? getError(action) : null
  });

  return _extends({}, state, defineProperty({}, apiName, next));
}), defineProperty(_handleActions, ACTION_FETCH_COMPLETE, function (state, action) {
  var apiName = getName(action);
  var api = state[apiName];
  var next = _extends({}, api, {
    isFetching: false,
    isInvalidated: false,
    lastResponse: getRespondedAt(action),
    data: getJSONResponse(action),
    error: null
  });

  return _extends({}, state, defineProperty({}, apiName, next));
}), defineProperty(_handleActions, ACTION_FETCH_FAILURE, function (state, action) {
  var apiName = getName(action);
  var api = state[apiName];
  var next = _extends({}, api, {
    isFetching: false,
    isInvalidated: true,
    error: getJSONResponse(action)
  });

  return _extends({}, state, defineProperty({}, apiName, next));
}), _handleActions));

var reducers = defineProperty({}, REDUCER_PATH, reducer);
var ACTIONS = {
  START: ACTION_FETCH_START,
  COMPLETE: ACTION_FETCH_COMPLETE,
  FAILURE: ACTION_FETCH_FAILURE
};

exports.middleware = middleware;
exports.makeFetchAction = makeFetchAction;
exports.reducers = reducers;
exports.ACTIONS = ACTIONS;
